package cjson

import "surdeus.su/core/xgo/v2"
import tjson "surdeus.su/core/xgo/v2/stdlib/json"

import "encoding/json"
import "io"
import "errors"
import "os"

type Decoder struct {
	xgo.ObjectImpl
	dec       *json.Decoder
	end       bool
	inputName string
}

func (d *Decoder) TypeName() string {
	return "cjson.Decoder"
}

func (d *Decoder) String() string {
	return "cjson.Decoder{...}"
}

func NewDecoder(args ...xgo.Object) (xgo.Object, error) {
	var inputName string
	if len(args) == 0 {
		inputName = StrStdin
	} else if len(args) > 1 {
		return nil, xgo.ErrWrongNumArguments
	} else {
		var ok bool
		inputObject := args[0]
		inputName, ok = xgo.ToString(inputObject)
		if !ok {
			return nil, xgo.ErrInvalidArgumentType{
				Name:     "first",
				Expected: "stringer",
				Found:    inputObject.TypeName(),
			}
		}
	}

	var reader io.Reader
	var err error
	switch {
	case inputName == StrStdin:
		reader = os.Stdin
	default:
		reader, err = os.Open(inputName)
		if err != nil {
			return nil, err
		}
	}

	ret := &Decoder{}
	ret.dec = json.NewDecoder(reader)

	return ret, nil
}

func (d *Decoder) IndexGet(
	index xgo.Object,
) (xgo.Object, error) {
	key, ok := xgo.ToString(index)
	if !ok {
		return nil, xgo.ErrInvalidIndexValueType
	}

	switch key {
	case "decode":
		return &xgo.UserFunction{
			Name:  "decode",
			Value: d.Decode,
		}, nil
	}

	// Nothing found.
	return nil, nil
}

func (d *Decoder) Decode(
	args ...xgo.Object,
) (xgo.Object, error) {
	if len(args) > 0 {
		return nil, xgo.ErrWrongNumArguments
	}
	if d.end {
		return nil, nil
	}

	var v any
	err := d.dec.Decode(&v)
	if err != nil {
		if errors.Is(err, io.EOF) {
			d.end = true
		} else {
			return nil, err
		}
	}

	bts, err := json.Marshal(v)
	if err != nil {
		return nil, err
	}
	return tjson.Decode(bts)
}
