package html

import "surdeus.su/core/xgo/v2"

var Module = map[string]xgo.Object{
	"Renderer": &xgo.BuiltinFunction{
		Name: "Renderer",
		Value: func(args ...xgo.Object) (xgo.Object, error) {
			ret := &Renderer{}
			return ret, nil
		},
	},
}

type Renderer struct {
	xgo.ObjectImpl
}

func (html *Renderer) TypeName() string {
	return "html.Renderer"
}

func (html *Renderer) String() string {
	return "html.Renderer(...)"
}

/*

html.div({
	id: "some-el-id",
	value: "shit value"
}).body(
	html.raw("cock "),
	html.strong("something")
)

*/

func (html *Renderer) IndexGet(
	index xgo.Object,
) (xgo.Object, error) {
	str, ok := xgo.ToString(index)
	if !ok {
		return nil, xgo.ErrInvalidIndexValueType
	}

	fn := func(args ...xgo.Object) (xgo.Object, error) {
		if len(args) > 1 {
			return nil, xgo.ErrWrongNumArguments
		}
		var arg xgo.Object
		if len(args) == 1 {
			arg = args[0]
		}

		if arg == nil {
			return &Element{
				Tag: str,
			}, nil
		}

		if can := arg.CanIterate(); !can {
			return nil, xgo.ErrInvalidArgumentType{
				Name:     "first",
				Expected: "iterable",
				Found:    arg.TypeName(),
			}
		}
		attr := map[string]string{}
		iter := arg.Iterate()
		for iter.Next() {
			key, val := iter.Key(), iter.Value()
			skey, ok := xgo.ToString(key)
			if !ok {
				return nil, xgo.ErrInvalidArgumentType{
					Name:     "attribute(key)",
					Expected: "stringer",
					Found:    key.TypeName(),
				}
			}
			sval, ok := xgo.ToString(val)
			if !ok {
				return nil, xgo.ErrInvalidArgumentType{
					Name:     "attribute(value)",
					Expected: "stringer",
					Found:    val.TypeName(),
				}
			}
			attr[skey] = sval
		}
		return &Element{
			Tag:  str,
			Attr: attr,
		}, nil
	}

	return &xgo.UserFunction{
		Name:  str,
		Value: fn,
	}, nil
}
