package http

import "surdeus.su/core/xgo/v2"
import "net/http"
import "bytes"
import tjson "surdeus.su/core/xgo/v2/stdlib/json"

var Module = map[string]xgo.Object{
	"get": &xgo.BuiltinFunction{
		Name: "get",
		Value: func(
			args ...xgo.Object,
		) (xgo.Object, error) {
			if len(args) != 1 {
				return nil, xgo.ErrWrongNumArguments
			}
			url, ok := xgo.ToString(args[0])
			if !ok {
				return nil, xgo.ErrInvalidArgumentType{
					Name:     "first",
					Expected: "string",
					Found:    args[0].TypeName(),
				}
			}
			resp, err := http.Get(url)
			if err != nil {
				return nil, err
			}
			return &Response{
				Response: resp,
			}, nil
		},
	},
	"post_json": &xgo.BuiltinFunction{
		Name: "post",
		Value: func(
			args ...xgo.Object,
		) (xgo.Object, error) {
			if len(args) < 2 {
				return nil, xgo.ErrWrongNumArguments
			}
			url, ok := xgo.ToString(args[0])
			if !ok {
				return nil, xgo.ErrInvalidArgumentType{
					Name:     "first",
					Expected: "string",
					Found:    args[0].TypeName(),
				}
			}

			var body bytes.Buffer
			ret, err := tjson.Encode(args[1])
			if err != nil {
				return nil, err
			}
			body.Write(ret)

			resp, err := http.Post(
				url, "application/json",
				&body,
			)
			if err != nil {
				return nil, err
			}
			return &Response{
				Response: resp,
			}, nil
		},
	},
}
