package stdlib

import (
	"math"

	"surdeus.su/core/xgo/v2"
)

var mathModule = map[string]xgo.Object{
	"e":                      &xgo.Float{Value: math.E},
	"pi":                     &xgo.Float{Value: math.Pi},
	"phi":                    &xgo.Float{Value: math.Phi},
	"sqrt2":                  &xgo.Float{Value: math.Sqrt2},
	"sqrtE":                  &xgo.Float{Value: math.SqrtE},
	"sqrtPi":                 &xgo.Float{Value: math.SqrtPi},
	"sqrtPhi":                &xgo.Float{Value: math.SqrtPhi},
	"ln2":                    &xgo.Float{Value: math.Ln2},
	"log2E":                  &xgo.Float{Value: math.Log2E},
	"ln10":                   &xgo.Float{Value: math.Ln10},
	"log10E":                 &xgo.Float{Value: math.Log10E},
	"maxFloat32":             &xgo.Float{Value: math.MaxFloat32},
	"smallestNonzeroFloat32": &xgo.Float{Value: math.SmallestNonzeroFloat32},
	"maxFloat64":             &xgo.Float{Value: math.MaxFloat64},
	"smallestNonzeroFloat64": &xgo.Float{Value: math.SmallestNonzeroFloat64},
	"maxInt":                 &xgo.Int{Value: math.MaxInt},
	"minInt":                 &xgo.Int{Value: math.MinInt},
	"maxInt8":                &xgo.Int{Value: math.MaxInt8},
	"minInt8":                &xgo.Int{Value: math.MinInt8},
	"maxInt16":               &xgo.Int{Value: math.MaxInt16},
	"minInt16":               &xgo.Int{Value: math.MinInt16},
	"maxInt32":               &xgo.Int{Value: math.MaxInt32},
	"minInt32":               &xgo.Int{Value: math.MinInt32},
	"maxInt64":               &xgo.Int{Value: math.MaxInt64},
	"minInt64":               &xgo.Int{Value: math.MinInt64},
	"abs": &xgo.UserFunction{
		Name:  "abs",
		Value: FuncAFRF(math.Abs),
	},
	"acos": &xgo.UserFunction{
		Name:  "acos",
		Value: FuncAFRF(math.Acos),
	},
	"acosh": &xgo.UserFunction{
		Name:  "acosh",
		Value: FuncAFRF(math.Acosh),
	},
	"asin": &xgo.UserFunction{
		Name:  "asin",
		Value: FuncAFRF(math.Asin),
	},
	"asinh": &xgo.UserFunction{
		Name:  "asinh",
		Value: FuncAFRF(math.Asinh),
	},
	"atan": &xgo.UserFunction{
		Name:  "atan",
		Value: FuncAFRF(math.Atan),
	},
	"atan2": &xgo.UserFunction{
		Name:  "atan2",
		Value: FuncAFFRF(math.Atan2),
	},
	"atanh": &xgo.UserFunction{
		Name:  "atanh",
		Value: FuncAFRF(math.Atanh),
	},
	"cbrt": &xgo.UserFunction{
		Name:  "cbrt",
		Value: FuncAFRF(math.Cbrt),
	},
	"ceil": &xgo.UserFunction{
		Name:  "ceil",
		Value: FuncAFRF(math.Ceil),
	},
	"copysign": &xgo.UserFunction{
		Name:  "copysign",
		Value: FuncAFFRF(math.Copysign),
	},
	"cos": &xgo.UserFunction{
		Name:  "cos",
		Value: FuncAFRF(math.Cos),
	},
	"cosh": &xgo.UserFunction{
		Name:  "cosh",
		Value: FuncAFRF(math.Cosh),
	},
	"dim": &xgo.UserFunction{
		Name:  "dim",
		Value: FuncAFFRF(math.Dim),
	},
	"erf": &xgo.UserFunction{
		Name:  "erf",
		Value: FuncAFRF(math.Erf),
	},
	"erfc": &xgo.UserFunction{
		Name:  "erfc",
		Value: FuncAFRF(math.Erfc),
	},
	"exp": &xgo.UserFunction{
		Name:  "exp",
		Value: FuncAFRF(math.Exp),
	},
	"exp2": &xgo.UserFunction{
		Name:  "exp2",
		Value: FuncAFRF(math.Exp2),
	},
	"expm1": &xgo.UserFunction{
		Name:  "expm1",
		Value: FuncAFRF(math.Expm1),
	},
	"floor": &xgo.UserFunction{
		Name:  "floor",
		Value: FuncAFRF(math.Floor),
	},
	"gamma": &xgo.UserFunction{
		Name:  "gamma",
		Value: FuncAFRF(math.Gamma),
	},
	"hypot": &xgo.UserFunction{
		Name:  "hypot",
		Value: FuncAFFRF(math.Hypot),
	},
	"ilogb": &xgo.UserFunction{
		Name:  "ilogb",
		Value: FuncAFRI(math.Ilogb),
	},
	"inf": &xgo.UserFunction{
		Name:  "inf",
		Value: FuncAIRF(math.Inf),
	},
	"is_inf": &xgo.UserFunction{
		Name:  "is_inf",
		Value: FuncAFIRB(math.IsInf),
	},
	"is_nan": &xgo.UserFunction{
		Name:  "is_nan",
		Value: FuncAFRB(math.IsNaN),
	},
	"j0": &xgo.UserFunction{
		Name:  "j0",
		Value: FuncAFRF(math.J0),
	},
	"j1": &xgo.UserFunction{
		Name:  "j1",
		Value: FuncAFRF(math.J1),
	},
	"jn": &xgo.UserFunction{
		Name:  "jn",
		Value: FuncAIFRF(math.Jn),
	},
	"ldexp": &xgo.UserFunction{
		Name:  "ldexp",
		Value: FuncAFIRF(math.Ldexp),
	},
	"log": &xgo.UserFunction{
		Name:  "log",
		Value: FuncAFRF(math.Log),
	},
	"log10": &xgo.UserFunction{
		Name:  "log10",
		Value: FuncAFRF(math.Log10),
	},
	"log1p": &xgo.UserFunction{
		Name:  "log1p",
		Value: FuncAFRF(math.Log1p),
	},
	"log2": &xgo.UserFunction{
		Name:  "log2",
		Value: FuncAFRF(math.Log2),
	},
	"logb": &xgo.UserFunction{
		Name:  "logb",
		Value: FuncAFRF(math.Logb),
	},
	"max": &xgo.UserFunction{
		Name:  "max",
		Value: FuncAFFRF(math.Max),
	},
	"min": &xgo.UserFunction{
		Name:  "min",
		Value: FuncAFFRF(math.Min),
	},
	"mod": &xgo.UserFunction{
		Name:  "mod",
		Value: FuncAFFRF(math.Mod),
	},
	"nan": &xgo.UserFunction{
		Name:  "nan",
		Value: FuncARF(math.NaN),
	},
	"nextafter": &xgo.UserFunction{
		Name:  "nextafter",
		Value: FuncAFFRF(math.Nextafter),
	},
	"pow": &xgo.UserFunction{
		Name:  "pow",
		Value: FuncAFFRF(math.Pow),
	},
	"pow10": &xgo.UserFunction{
		Name:  "pow10",
		Value: FuncAIRF(math.Pow10),
	},
	"remainder": &xgo.UserFunction{
		Name:  "remainder",
		Value: FuncAFFRF(math.Remainder),
	},
	"signbit": &xgo.UserFunction{
		Name:  "signbit",
		Value: FuncAFRB(math.Signbit),
	},
	"sin": &xgo.UserFunction{
		Name:  "sin",
		Value: FuncAFRF(math.Sin),
	},
	"sinh": &xgo.UserFunction{
		Name:  "sinh",
		Value: FuncAFRF(math.Sinh),
	},
	"sqrt": &xgo.UserFunction{
		Name:  "sqrt",
		Value: FuncAFRF(math.Sqrt),
	},
	"tan": &xgo.UserFunction{
		Name:  "tan",
		Value: FuncAFRF(math.Tan),
	},
	"tanh": &xgo.UserFunction{
		Name:  "tanh",
		Value: FuncAFRF(math.Tanh),
	},
	"trunc": &xgo.UserFunction{
		Name:  "trunc",
		Value: FuncAFRF(math.Trunc),
	},
	"y0": &xgo.UserFunction{
		Name:  "y0",
		Value: FuncAFRF(math.Y0),
	},
	"y1": &xgo.UserFunction{
		Name:  "y1",
		Value: FuncAFRF(math.Y1),
	},
	"yn": &xgo.UserFunction{
		Name:  "yn",
		Value: FuncAIFRF(math.Yn),
	},
}
