package stdlib

import (
	"os/exec"

	"surdeus.su/core/xgo/v2"
)

func makeOSExecCommand(cmd *exec.Cmd) *xgo.ImmutableMap {
	return &xgo.ImmutableMap{
		Value: map[string]xgo.Object{
			// combined_output() => bytes/error
			"combined_output": &xgo.UserFunction{
				Name:  "combined_output",
				Value: FuncARYE(cmd.CombinedOutput),
			},
			// output() => bytes/error
			"output": &xgo.UserFunction{
				Name:  "output",
				Value: FuncARYE(cmd.Output),
			}, //
			// run() => error
			"run": &xgo.UserFunction{
				Name:  "run",
				Value: FuncARE(cmd.Run),
			}, //
			// start() => error
			"start": &xgo.UserFunction{
				Name:  "start",
				Value: FuncARE(cmd.Start),
			}, //
			// wait() => error
			"wait": &xgo.UserFunction{
				Name:  "wait",
				Value: FuncARE(cmd.Wait),
			}, //
			// set_path(path string)
			"set_path": &xgo.UserFunction{
				Name: "set_path",
				Value: func(args ...xgo.Object) (xgo.Object, error) {
					if len(args) != 1 {
						return nil, xgo.ErrWrongNumArguments
					}
					s1, ok := xgo.ToString(args[0])
					if !ok {
						return nil, xgo.ErrInvalidArgumentType{
							Name:     "first",
							Expected: "string(compatible)",
							Found:    args[0].TypeName(),
						}
					}
					cmd.Path = s1
					return xgo.UndefinedValue, nil
				},
			},
			// set_dir(dir string)
			"set_dir": &xgo.UserFunction{
				Name: "set_dir",
				Value: func(args ...xgo.Object) (xgo.Object, error) {
					if len(args) != 1 {
						return nil, xgo.ErrWrongNumArguments
					}
					s1, ok := xgo.ToString(args[0])
					if !ok {
						return nil, xgo.ErrInvalidArgumentType{
							Name:     "first",
							Expected: "string(compatible)",
							Found:    args[0].TypeName(),
						}
					}
					cmd.Dir = s1
					return xgo.UndefinedValue, nil
				},
			},
			// set_env(env array(string))
			"set_env": &xgo.UserFunction{
				Name: "set_env",
				Value: func(args ...xgo.Object) (xgo.Object, error) {
					if len(args) != 1 {
						return nil, xgo.ErrWrongNumArguments
					}

					var env []string
					var err error
					switch arg0 := args[0].(type) {
					case *xgo.Array:
						env, err = stringArray(arg0.Value, "first")
						if err != nil {
							return nil, err
						}
					case *xgo.ImmutableArray:
						env, err = stringArray(arg0.Value, "first")
						if err != nil {
							return nil, err
						}
					default:
						return nil, xgo.ErrInvalidArgumentType{
							Name:     "first",
							Expected: "array",
							Found:    arg0.TypeName(),
						}
					}
					cmd.Env = env
					return xgo.UndefinedValue, nil
				},
			},
			// process() => imap(process)
			"process": &xgo.UserFunction{
				Name: "process",
				Value: func(args ...xgo.Object) (xgo.Object, error) {
					if len(args) != 0 {
						return nil, xgo.ErrWrongNumArguments
					}
					return makeOSProcess(cmd.Process), nil
				},
			},
		},
	}
}
