package stdlib

import (
	"os"
	"syscall"

	"surdeus.su/core/xgo/v2"
)

func makeOSProcessState(state *os.ProcessState) *xgo.ImmutableMap {
	return &xgo.ImmutableMap{
		Value: map[string]xgo.Object{
			"exited": &xgo.UserFunction{
				Name:  "exited",
				Value: FuncARB(state.Exited),
			},
			"pid": &xgo.UserFunction{
				Name:  "pid",
				Value: FuncARI(state.Pid),
			},
			"string": &xgo.UserFunction{
				Name:  "string",
				Value: FuncARS(state.String),
			},
			"success": &xgo.UserFunction{
				Name:  "success",
				Value: FuncARB(state.Success),
			},
		},
	}
}

func makeOSProcess(proc *os.Process) *xgo.ImmutableMap {
	return &xgo.ImmutableMap{
		Value: map[string]xgo.Object{
			"kill": &xgo.UserFunction{
				Name:  "kill",
				Value: FuncARE(proc.Kill),
			},
			"release": &xgo.UserFunction{
				Name:  "release",
				Value: FuncARE(proc.Release),
			},
			"signal": &xgo.UserFunction{
				Name: "signal",
				Value: func(args ...xgo.Object) (xgo.Object, error) {
					if len(args) != 1 {
						return nil, xgo.ErrWrongNumArguments
					}
					i1, ok := xgo.ToInt64(args[0])
					if !ok {
						return nil, xgo.ErrInvalidArgumentType{
							Name:     "first",
							Expected: "int(compatible)",
							Found:    args[0].TypeName(),
						}
					}
					return wrapError(proc.Signal(syscall.Signal(i1))), nil
				},
			},
			"wait": &xgo.UserFunction{
				Name: "wait",
				Value: func(args ...xgo.Object) (xgo.Object, error) {
					if len(args) != 0 {
						return nil, xgo.ErrWrongNumArguments
					}
					state, err := proc.Wait()
					if err != nil {
						return wrapError(err), nil
					}
					return makeOSProcessState(state), nil
				},
			},
		},
	}
}
