package paths

import (
	"surdeus.su/core/xgo/v2"
	"path"
)

var Module = map[string]xgo.Object{
	"base": &xgo.UserFunction{
		Name:  "base",
		Value: wrapOneArg(path.Base),
	},
	"ext": &xgo.UserFunction{
		Name:  "ext",
		Value: wrapOneArg(path.Ext),
	},
	"dir": &xgo.UserFunction{
		Name:  "dir",
		Value: wrapOneArg(path.Dir),
	},
	"clean": &xgo.UserFunction{
		Name:  "clean",
		Value: wrapOneArg(path.Clean),
	},
}

func wrapOneArg(
	fn func(string) string,
) func(...xgo.Object) (xgo.Object, error) {
	return func(args ...xgo.Object) (xgo.Object, error) {
		if len(args) != 1 {
			return nil, xgo.ErrWrongNumArguments
		}
		str, ok := xgo.ToString(args[0])
		if !ok {
			return nil, xgo.ErrInvalidArgumentType{
				Name:     "first",
				Expected: "string(compatible)",
				Found:    args[0].TypeName(),
			}
		}
		return &xgo.String{
			Value: fn(str),
		}, nil
	}
}
