package stdlib

import (
	"math/rand"

	"surdeus.su/core/xgo/v2"
)

var randModule = map[string]xgo.Object{
	"int": &xgo.UserFunction{
		Name:  "int",
		Value: FuncARI64(rand.Int63),
	},
	"float": &xgo.UserFunction{
		Name:  "float",
		Value: FuncARF(rand.Float64),
	},
	"intn": &xgo.UserFunction{
		Name:  "intn",
		Value: FuncAI64RI64(rand.Int63n),
	},
	"exp_float": &xgo.UserFunction{
		Name:  "exp_float",
		Value: FuncARF(rand.ExpFloat64),
	},
	"norm_float": &xgo.UserFunction{
		Name:  "norm_float",
		Value: FuncARF(rand.NormFloat64),
	},
	"perm": &xgo.UserFunction{
		Name:  "perm",
		Value: FuncAIRIs(rand.Perm),
	},
	"seed": &xgo.UserFunction{
		Name:  "seed",
		Value: FuncAI64R(rand.Seed),
	},
	"read": &xgo.UserFunction{
		Name: "read",
		Value: func(args ...xgo.Object) (ret xgo.Object, err error) {
			if len(args) != 1 {
				return nil, xgo.ErrWrongNumArguments
			}
			y1, ok := args[0].(*xgo.Bytes)
			if !ok {
				return nil, xgo.ErrInvalidArgumentType{
					Name:     "first",
					Expected: "bytes",
					Found:    args[0].TypeName(),
				}
			}
			res, err := rand.Read(y1.Value)
			if err != nil {
				ret = wrapError(err)
				return
			}
			return &xgo.Int{Value: int64(res)}, nil
		},
	},
	"rand": &xgo.UserFunction{
		Name: "rand",
		Value: func(args ...xgo.Object) (xgo.Object, error) {
			if len(args) != 1 {
				return nil, xgo.ErrWrongNumArguments
			}
			i1, ok := xgo.ToInt64(args[0])
			if !ok {
				return nil, xgo.ErrInvalidArgumentType{
					Name:     "first",
					Expected: "int(compatible)",
					Found:    args[0].TypeName(),
				}
			}
			src := rand.NewSource(i1)
			return randRand(rand.New(src)), nil
		},
	},
}

func randRand(r *rand.Rand) *xgo.ImmutableMap {
	return &xgo.ImmutableMap{
		Value: map[string]xgo.Object{
			"int": &xgo.UserFunction{
				Name:  "int",
				Value: FuncARI64(r.Int63),
			},
			"float": &xgo.UserFunction{
				Name:  "float",
				Value: FuncARF(r.Float64),
			},
			"intn": &xgo.UserFunction{
				Name:  "intn",
				Value: FuncAI64RI64(r.Int63n),
			},
			"exp_float": &xgo.UserFunction{
				Name:  "exp_float",
				Value: FuncARF(r.ExpFloat64),
			},
			"norm_float": &xgo.UserFunction{
				Name:  "norm_float",
				Value: FuncARF(r.NormFloat64),
			},
			"perm": &xgo.UserFunction{
				Name:  "perm",
				Value: FuncAIRIs(r.Perm),
			},
			"seed": &xgo.UserFunction{
				Name:  "seed",
				Value: FuncAI64R(r.Seed),
			},
			"read": &xgo.UserFunction{
				Name: "read",
				Value: func(args ...xgo.Object) (
					ret xgo.Object,
					err error,
				) {
					if len(args) != 1 {
						return nil, xgo.ErrWrongNumArguments
					}
					y1, ok := args[0].(*xgo.Bytes)
					if !ok {
						return nil, xgo.ErrInvalidArgumentType{
							Name:     "first",
							Expected: "bytes",
							Found:    args[0].TypeName(),
						}
					}
					res, err := r.Read(y1.Value)
					if err != nil {
						ret = wrapError(err)
						return
					}
					return &xgo.Int{Value: int64(res)}, nil
				},
			},
		},
	}
}
