package stdlib

import (
	"time"

	"surdeus.su/core/xgo/v2"
)

var timesModule = map[string]xgo.Object{
	"format_ansic":        &xgo.String{Value: time.ANSIC},
	"format_unix_date":    &xgo.String{Value: time.UnixDate},
	"format_ruby_date":    &xgo.String{Value: time.RubyDate},
	"format_rfc822":       &xgo.String{Value: time.RFC822},
	"format_rfc822z":      &xgo.String{Value: time.RFC822Z},
	"format_rfc850":       &xgo.String{Value: time.RFC850},
	"format_rfc1123":      &xgo.String{Value: time.RFC1123},
	"format_rfc1123z":     &xgo.String{Value: time.RFC1123Z},
	"format_rfc3339":      &xgo.String{Value: time.RFC3339},
	"format_rfc3339_nano": &xgo.String{Value: time.RFC3339Nano},
	"format_kitchen":      &xgo.String{Value: time.Kitchen},
	"format_stamp":        &xgo.String{Value: time.Stamp},
	"format_stamp_milli":  &xgo.String{Value: time.StampMilli},
	"format_stamp_micro":  &xgo.String{Value: time.StampMicro},
	"format_stamp_nano":   &xgo.String{Value: time.StampNano},
	"nanosecond":          &xgo.Int{Value: int64(time.Nanosecond)},
	"microsecond":         &xgo.Int{Value: int64(time.Microsecond)},
	"millisecond":         &xgo.Int{Value: int64(time.Millisecond)},
	"second":              &xgo.Int{Value: int64(time.Second)},
	"minute":              &xgo.Int{Value: int64(time.Minute)},
	"hour":                &xgo.Int{Value: int64(time.Hour)},
	"january":             &xgo.Int{Value: int64(time.January)},
	"february":            &xgo.Int{Value: int64(time.February)},
	"march":               &xgo.Int{Value: int64(time.March)},
	"april":               &xgo.Int{Value: int64(time.April)},
	"may":                 &xgo.Int{Value: int64(time.May)},
	"june":                &xgo.Int{Value: int64(time.June)},
	"july":                &xgo.Int{Value: int64(time.July)},
	"august":              &xgo.Int{Value: int64(time.August)},
	"september":           &xgo.Int{Value: int64(time.September)},
	"october":             &xgo.Int{Value: int64(time.October)},
	"november":            &xgo.Int{Value: int64(time.November)},
	"december":            &xgo.Int{Value: int64(time.December)},
	"sleep": &xgo.UserFunction{
		Name:  "sleep",
		Value: timesSleep,
	}, // sleep(int)
	"parse_duration": &xgo.UserFunction{
		Name:  "parse_duration",
		Value: timesParseDuration,
	}, // parse_duration(str) => int
	"since": &xgo.UserFunction{
		Name:  "since",
		Value: timesSince,
	}, // since(time) => int
	"until": &xgo.UserFunction{
		Name:  "until",
		Value: timesUntil,
	}, // until(time) => int
	"duration_hours": &xgo.UserFunction{
		Name:  "duration_hours",
		Value: timesDurationHours,
	}, // duration_hours(int) => float
	"duration_minutes": &xgo.UserFunction{
		Name:  "duration_minutes",
		Value: timesDurationMinutes,
	}, // duration_minutes(int) => float
	"duration_nanoseconds": &xgo.UserFunction{
		Name:  "duration_nanoseconds",
		Value: timesDurationNanoseconds,
	}, // duration_nanoseconds(int) => int
	"duration_seconds": &xgo.UserFunction{
		Name:  "duration_seconds",
		Value: timesDurationSeconds,
	}, // duration_seconds(int) => float
	"duration_string": &xgo.UserFunction{
		Name:  "duration_string",
		Value: timesDurationString,
	}, // duration_string(int) => string
	"month_string": &xgo.UserFunction{
		Name:  "month_string",
		Value: timesMonthString,
	}, // month_string(int) => string
	"date": &xgo.UserFunction{
		Name:  "date",
		Value: timesDate,
	}, // date(year, month, day, hour, min, sec, nsec) => time
	"now": &xgo.UserFunction{
		Name:  "now",
		Value: timesNow,
	}, // now() => time
	"parse": &xgo.UserFunction{
		Name:  "parse",
		Value: timesParse,
	}, // parse(format, str) => time
	"unix": &xgo.UserFunction{
		Name:  "unix",
		Value: timesUnix,
	}, // unix(sec, nsec) => time
	"add": &xgo.UserFunction{
		Name:  "add",
		Value: timesAdd,
	}, // add(time, int) => time
	"add_date": &xgo.UserFunction{
		Name:  "add_date",
		Value: timesAddDate,
	}, // add_date(time, years, months, days) => time
	"sub": &xgo.UserFunction{
		Name:  "sub",
		Value: timesSub,
	}, // sub(t time, u time) => int
	"after": &xgo.UserFunction{
		Name:  "after",
		Value: timesAfter,
	}, // after(t time, u time) => bool
	"before": &xgo.UserFunction{
		Name:  "before",
		Value: timesBefore,
	}, // before(t time, u time) => bool
	"time_year": &xgo.UserFunction{
		Name:  "time_year",
		Value: timesTimeYear,
	}, // time_year(time) => int
	"time_month": &xgo.UserFunction{
		Name:  "time_month",
		Value: timesTimeMonth,
	}, // time_month(time) => int
	"time_day": &xgo.UserFunction{
		Name:  "time_day",
		Value: timesTimeDay,
	}, // time_day(time) => int
	"time_weekday": &xgo.UserFunction{
		Name:  "time_weekday",
		Value: timesTimeWeekday,
	}, // time_weekday(time) => int
	"time_hour": &xgo.UserFunction{
		Name:  "time_hour",
		Value: timesTimeHour,
	}, // time_hour(time) => int
	"time_minute": &xgo.UserFunction{
		Name:  "time_minute",
		Value: timesTimeMinute,
	}, // time_minute(time) => int
	"time_second": &xgo.UserFunction{
		Name:  "time_second",
		Value: timesTimeSecond,
	}, // time_second(time) => int
	"time_nanosecond": &xgo.UserFunction{
		Name:  "time_nanosecond",
		Value: timesTimeNanosecond,
	}, // time_nanosecond(time) => int
	"time_unix": &xgo.UserFunction{
		Name:  "time_unix",
		Value: timesTimeUnix,
	}, // time_unix(time) => int
	"time_unix_nano": &xgo.UserFunction{
		Name:  "time_unix_nano",
		Value: timesTimeUnixNano,
	}, // time_unix_nano(time) => int
	"time_format": &xgo.UserFunction{
		Name:  "time_format",
		Value: timesTimeFormat,
	}, // time_format(time, format) => string
	"time_location": &xgo.UserFunction{
		Name:  "time_location",
		Value: timesTimeLocation,
	}, // time_location(time) => string
	"time_string": &xgo.UserFunction{
		Name:  "time_string",
		Value: timesTimeString,
	}, // time_string(time) => string
	"is_zero": &xgo.UserFunction{
		Name:  "is_zero",
		Value: timesIsZero,
	}, // is_zero(time) => bool
	"to_local": &xgo.UserFunction{
		Name:  "to_local",
		Value: timesToLocal,
	}, // to_local(time) => time
	"to_utc": &xgo.UserFunction{
		Name:  "to_utc",
		Value: timesToUTC,
	}, // to_utc(time) => time
	"in_location": &xgo.UserFunction{
		Name:  "in_location",
		Value: timesInLocation,
	}, // in_location(time, location) => time
}

func timesSleep(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	i1, ok := xgo.ToInt64(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "int(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	time.Sleep(time.Duration(i1))
	ret = xgo.UndefinedValue

	return
}

func timesParseDuration(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	s1, ok := xgo.ToString(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "string(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	dur, err := time.ParseDuration(s1)
	if err != nil {
		ret = wrapError(err)
		return
	}

	ret = &xgo.Int{Value: int64(dur)}

	return
}

func timesSince(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(time.Since(t1))}

	return
}

func timesUntil(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(time.Until(t1))}

	return
}

func timesDurationHours(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	i1, ok := xgo.ToInt64(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "int(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Float{Value: time.Duration(i1).Hours()}

	return
}

func timesDurationMinutes(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	i1, ok := xgo.ToInt64(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "int(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Float{Value: time.Duration(i1).Minutes()}

	return
}

func timesDurationNanoseconds(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	i1, ok := xgo.ToInt64(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "int(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: time.Duration(i1).Nanoseconds()}

	return
}

func timesDurationSeconds(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	i1, ok := xgo.ToInt64(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "int(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Float{Value: time.Duration(i1).Seconds()}

	return
}

func timesDurationString(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	i1, ok := xgo.ToInt64(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "int(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.String{Value: time.Duration(i1).String()}

	return
}

func timesMonthString(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	i1, ok := xgo.ToInt64(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "int(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.String{Value: time.Month(i1).String()}

	return
}

func timesDate(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) < 7 || len(args) > 8 {
		err = xgo.ErrWrongNumArguments
		return
	}

	i1, ok := xgo.ToInt(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "int(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}
	i2, ok := xgo.ToInt(args[1])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "second",
			Expected: "int(compatible)",
			Found:    args[1].TypeName(),
		}
		return
	}
	i3, ok := xgo.ToInt(args[2])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "third",
			Expected: "int(compatible)",
			Found:    args[2].TypeName(),
		}
		return
	}
	i4, ok := xgo.ToInt(args[3])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "fourth",
			Expected: "int(compatible)",
			Found:    args[3].TypeName(),
		}
		return
	}
	i5, ok := xgo.ToInt(args[4])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "fifth",
			Expected: "int(compatible)",
			Found:    args[4].TypeName(),
		}
		return
	}
	i6, ok := xgo.ToInt(args[5])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "sixth",
			Expected: "int(compatible)",
			Found:    args[5].TypeName(),
		}
		return
	}
	i7, ok := xgo.ToInt(args[6])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "seventh",
			Expected: "int(compatible)",
			Found:    args[6].TypeName(),
		}
		return
	}

	var loc *time.Location
	if len(args) == 8 {
		i8, ok := xgo.ToString(args[7])
		if !ok {
			err = xgo.ErrInvalidArgumentType{
				Name:     "eighth",
				Expected: "string(compatible)",
				Found:    args[7].TypeName(),
			}
			return
		}
		loc, err = time.LoadLocation(i8)
		if err != nil {
			ret = wrapError(err)
			return
		}
	} else {
		loc = time.Now().Location()
	}

	ret = &xgo.Time{
		Value: time.Date(i1,
			time.Month(i2), i3, i4, i5, i6, i7, loc),
	}

	return
}

func timesNow(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 0 {
		err = xgo.ErrWrongNumArguments
		return
	}

	ret = &xgo.Time{Value: time.Now()}

	return
}

func timesParse(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 2 {
		err = xgo.ErrWrongNumArguments
		return
	}

	s1, ok := xgo.ToString(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "string(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	s2, ok := xgo.ToString(args[1])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "second",
			Expected: "string(compatible)",
			Found:    args[1].TypeName(),
		}
		return
	}

	parsed, err := time.Parse(s1, s2)
	if err != nil {
		ret = wrapError(err)
		return
	}

	ret = &xgo.Time{Value: parsed}

	return
}

func timesUnix(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 2 {
		err = xgo.ErrWrongNumArguments
		return
	}

	i1, ok := xgo.ToInt64(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "int(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	i2, ok := xgo.ToInt64(args[1])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "second",
			Expected: "int(compatible)",
			Found:    args[1].TypeName(),
		}
		return
	}

	ret = &xgo.Time{Value: time.Unix(i1, i2)}

	return
}

func timesAdd(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 2 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	i2, ok := xgo.ToInt64(args[1])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "second",
			Expected: "int(compatible)",
			Found:    args[1].TypeName(),
		}
		return
	}

	ret = &xgo.Time{Value: t1.Add(time.Duration(i2))}

	return
}

func timesSub(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 2 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	t2, ok := xgo.ToTime(args[1])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "second",
			Expected: "time(compatible)",
			Found:    args[1].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(t1.Sub(t2))}

	return
}

func timesAddDate(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 4 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	i2, ok := xgo.ToInt(args[1])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "second",
			Expected: "int(compatible)",
			Found:    args[1].TypeName(),
		}
		return
	}

	i3, ok := xgo.ToInt(args[2])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "third",
			Expected: "int(compatible)",
			Found:    args[2].TypeName(),
		}
		return
	}

	i4, ok := xgo.ToInt(args[3])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "fourth",
			Expected: "int(compatible)",
			Found:    args[3].TypeName(),
		}
		return
	}

	ret = &xgo.Time{Value: t1.AddDate(i2, i3, i4)}

	return
}

func timesAfter(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 2 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	t2, ok := xgo.ToTime(args[1])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "second",
			Expected: "time(compatible)",
			Found:    args[1].TypeName(),
		}
		return
	}

	if t1.After(t2) {
		ret = xgo.TrueValue
	} else {
		ret = xgo.FalseValue
	}

	return
}

func timesBefore(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 2 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	t2, ok := xgo.ToTime(args[1])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "second",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	if t1.Before(t2) {
		ret = xgo.TrueValue
	} else {
		ret = xgo.FalseValue
	}

	return
}

func timesTimeYear(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(t1.Year())}

	return
}

func timesTimeMonth(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(t1.Month())}

	return
}

func timesTimeDay(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(t1.Day())}

	return
}

func timesTimeWeekday(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(t1.Weekday())}

	return
}

func timesTimeHour(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(t1.Hour())}

	return
}

func timesTimeMinute(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(t1.Minute())}

	return
}

func timesTimeSecond(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(t1.Second())}

	return
}

func timesTimeNanosecond(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: int64(t1.Nanosecond())}

	return
}

func timesTimeUnix(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: t1.Unix()}

	return
}

func timesTimeUnixNano(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Int{Value: t1.UnixNano()}

	return
}

func timesTimeFormat(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 2 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	s2, ok := xgo.ToString(args[1])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "second",
			Expected: "string(compatible)",
			Found:    args[1].TypeName(),
		}
		return
	}

	s := t1.Format(s2)
	if len(s) > xgo.MaxStringLen {

		return nil, xgo.ErrStringLimit
	}

	ret = &xgo.String{Value: s}

	return
}

func timesIsZero(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	if t1.IsZero() {
		ret = xgo.TrueValue
	} else {
		ret = xgo.FalseValue
	}

	return
}

func timesToLocal(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Time{Value: t1.Local()}

	return
}

func timesToUTC(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.Time{Value: t1.UTC()}

	return
}

func timesTimeLocation(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.String{Value: t1.Location().String()}

	return
}

func timesInLocation(args ...xgo.Object) (
	ret xgo.Object,
	err error,
) {
	if len(args) != 2 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	s2, ok := xgo.ToString(args[1])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "second",
			Expected: "string(compatible)",
			Found:    args[1].TypeName(),
		}
		return
	}

	location, err := time.LoadLocation(s2)
	if err != nil {
		ret = wrapError(err)
		return
	}

	ret = &xgo.Time{Value: t1.In(location)}

	return
}

func timesTimeString(args ...xgo.Object) (ret xgo.Object, err error) {
	if len(args) != 1 {
		err = xgo.ErrWrongNumArguments
		return
	}

	t1, ok := xgo.ToTime(args[0])
	if !ok {
		err = xgo.ErrInvalidArgumentType{
			Name:     "first",
			Expected: "time(compatible)",
			Found:    args[0].TypeName(),
		}
		return
	}

	ret = &xgo.String{Value: t1.String()}

	return
}
