package url

import (
	"surdeus.su/core/xgo/v2"
	"net/url"
	"fmt"
)

var _ = xgo.Object(&Values{})

type Values struct {
	xgo.ObjectImpl
	url.Values
}

func (vs *Values) TypeName() string {
	return "*url.Values"
}

func (vs *Values) String() string {
	return fmt.Sprintf("%v", vs.Values)
}

func (vs *Values) IndexGet(
	index xgo.Object,
) (xgo.Object, error) {
	key, ok := xgo.ToString(index)
	if !ok {
		return nil, xgo.ErrInvalidIndexValueType
	}

	val, ok := vs.Values[key]
	if !ok {
		return nil, nil
	}

	arr := make([]xgo.Object, len(val))
	for i, v := range val {
		arr[i], _ = xgo.FromInterface(v)
	}

	return &xgo.Array{Value: arr}, nil
}

type URL struct {
	xgo.ObjectImpl
	*url.URL
}

func (u *URL) TypeName() string {
	return "<URL>"
}

func (u *URL) String() string {
	return u.URL.String()
}

func (u *URL) IndexGet(
	index xgo.Object,
) (xgo.Object, error) {
	key, ok := xgo.ToString(index)
	if !ok {
		return nil, xgo.ErrInvalidIndexValueType
	}

	switch key {
	case "path":
		return xgo.FromInterface(u.Path)
	case "query":
		return &Values{
			Values: u.Query(),
		}, nil
	}

	// Nothing found.
	return nil, nil
}
